clear;clc;close all;
% Read Lytro data, please download the data at
% http://lightfields.stanford.edu/
fileName='.\Data\XXXX.png';
im=imread(fileName);
numImgsX = 14;
numImgsY = 14;
% Extract sub-aperture images
inputImg = im2double(im);
h = size(inputImg, 1) / numImgsY;
w = size(inputImg, 2) / numImgsX;
LFGT = zeros(h, w, 3, numImgsY, numImgsX);
inputImg=AdjustTone(inputImg,2.25); % Adjust the tone of the Lytro data
for ax = 1 : numImgsX
    for ay = 1 : numImgsY
        im=inputImg(ay:numImgsY:end, ax:numImgsX:end, :);
        LFGT(:, :, :, ay, ax) =im;
    end
end
LFGT=uint8(LFGT(:,:,:,5:11,5:11)*255);
[H,W,C,LFDimOut,~]=size(LFGT);
% Angularly downsample. We encourage at least 4x4 views are used as input
% due to the bicubic upsampling.
LFIn=LFGT(:,:,:,1:2:end,1:2:end);
LFDimIn=size(LFIn,4);
figure(1);imshow(LFIn(:,:,:,1,1));
%% Parameters setting
upScale=fix(LFDimOut/LFDimIn)+1;
sizeKernel=3;
if sizeKernel>1
    load BlurKernel
    blurKernel=imresize(kernel,[3,sizeKernel],'bicubic');
    blurKernel=blurKernel/(sum(blurKernel(:)));
else
    blurKernel=[0,1,0];
end
% Parameters for deblur
weight_ring = 1;
lambda_tv = 0.001;
lambda_l0 = 1e-4;
LFOut=uint8(zeros(H,W,C,LFDimOut,LFDimOut));
%% Iteration 1: Row Interpolation
fprintf('Begin iteration 1: Row Interpolation.\n');
for row=1:LFDimIn
    SliceIn=LFIn(:,:,:,row,:);
    SliceIn=permute(SliceIn,[1,2,3,5,4]);
    
    % "Blur-Restoration" step
    SliceOut=fun_InterpKernel(SliceIn,LFDimOut,LFDimIn,blurKernel);
    
    % "Deblur" step
    parfor i=1:LFDimOut
        im=double(SliceOut(:,:,:,i))/255;
        if sizeKernel>1
            im = ringing_artifacts_removal(im, blurKernel, lambda_tv, lambda_l0, weight_ring);
        end
        SliceOut(:,:,:,i)=uint8(im*255);
    end
    SliceOut=permute(SliceOut,[1,2,3,5,4]);
    LFOut(:,:,:,(row-1)*upScale+1,:)=SliceOut;
    fprintf('  Row %d has been processed.\n',row);
end
%% Iteration 1: Colume Interpolation
fprintf('Begin iteration 1: Colume Interpolation.\n');
for colume=1:LFDimIn
    SliceIn=LFIn(:,:,:,:,colume);
    SliceIn=permute(SliceIn,[2,1,3,4]);
    
    % "Blur-Restoration" step
    SliceOut=fun_InterpKernel(SliceIn,LFDimOut,LFDimIn,blurKernel);
    
    % "Deblur" step
    parfor i=1:LFDimOut
        im=double(SliceOut(:,:,:,i))/255;
        if sizeKernel>1
            im = ringing_artifacts_removal(im, blurKernel, lambda_tv, lambda_l0, weight_ring);
        end
        SliceOut(:,:,:,i)=uint8(im*255);
    end
    SliceOut=permute(SliceOut,[2,1,3,4]);
    LFOut(:,:,:,:,(colume-1)*upScale+1)=SliceOut;
    fprintf('  Colume %d has been processed.\n',colume);
end
%% Iteration 2: Row Interpolation
fprintf('Begin iteration 2: Row Interpolation.\n');
idxInterp=1:LFDimOut;
idxInterp=setdiff(idxInterp,([1:LFDimIn]-1)*upScale+1);
for row=idxInterp
    SliceIn=LFOut(:,:,:,row,1:upScale:LFDimOut);
    SliceIn=permute(SliceIn,[1,2,3,5,4]);
    
    % "Blur-Restoration" step
    SliceOut=fun_InterpKernel(SliceIn,LFDimOut,LFDimIn,blurKernel);
    
    % "Deblur" step
    parfor i=1:LFDimOut
        im=double(SliceOut(:,:,:,i))/255;
        if sizeKernel>1
            im = ringing_artifacts_removal(im, blurKernel, lambda_tv, lambda_l0, weight_ring);
        end
        SliceOut(:,:,:,i)=uint8(im*255);
    end
    SliceOut=permute(SliceOut,[1,2,3,5,4]);
    LFOut(:,:,:,row,:)=SliceOut;
    fprintf('  Row %d has been processed.\n',row);
end
%% Display
for i=1:LFDimOut
    if mod(i,2)==1
        displaySequence=1:LFDimOut;
    else
        displaySequence=LFDimOut:-1:1;
    end
    for j=displaySequence
        figure(1);imshow(LFOut(:,:,:,i,j));
        figure(1);imshow(LFOut(:,:,:,i,j));
    end
end
%% Evaluation
PSNR=0;
SSIM=0;
k=0;
for i=1:LFDimOut
    for j=1:LFDimOut
        k=k+1;
        [psnr,ssim]=compute_psnr(LFOut(:,:,:,i,j),LFGT(:,:,:,i,j),0);
        PSNR=PSNR+psnr;
        SSIM=SSIM+ssim;
    end
end
PSNR=PSNR/k;
SSIM=SSIM/k;